@@ -126,7 +126,8 @@ static int ai_goldequiv_tech(struct player *pplayer, Tech_type_id tech)
 {
   int bulbs, tech_want, worth;
 
-  if (get_invention(pplayer, tech) == TECH_KNOWN) {
+  if (get_invention(pplayer, tech) == TECH_KNOWN
+      || !tech_is_available(pplayer, tech)) {
     return 0;
   }
   bulbs = total_bulbs_required_for_goal(pplayer, tech) * 3;
@@ -423,6 +424,7 @@ static int ai_goldequiv_clause(struct player *pplayer,
       notify(aplayer, _("*%s (AI)* I do not know the city you mention."),
              player_name(pplayer));
       worth = 0;
+      DIPLO_LOG(LOG_DEBUG, pplayer, aplayer, "city destroyed during negotiations");
     } else if (give) {
       /* AI must be crazy to trade away its cities */
       worth -= city_gold_worth(offer);
@@ -438,8 +440,10 @@ static int ai_goldequiv_clause(struct player *pplayer,
     } else {
       worth = city_gold_worth(offer);
     }
-    DIPLO_LOG(LOG_DEBUG, pplayer, aplayer, "worth of %s is %d", 
-              city_name(offer), worth);
+    if (offer != NULL) {
+      DIPLO_LOG(LOG_DEBUG, pplayer, aplayer, "worth of %s is %d", 
+		city_name(offer), worth);
+    }
     break;
   }
 
@@ -968,7 +972,9 @@ static void suggest_tech_exchange(struct player* player1,
   bool is_dangerous;
     
   tech_type_iterate(tech) {
-    if (tech == A_NONE) {
+    if (tech == A_NONE 
+        || !tech_is_available(player2, tech)
+        || !tech_is_available(player1, tech)) {
       worth[tech] = 0;
       continue;
     }
@@ -1037,10 +1043,12 @@ static void ai_share(struct player *pplayer, struct player *aplayer)
   if (players_on_same_team(pplayer, aplayer)) {
     for (index = A_FIRST; index < game.control.num_tech_types; index++) {
       if ((get_invention(pplayer, index) != TECH_KNOWN)
-          && (get_invention(aplayer, index) == TECH_KNOWN)) {
+          && (get_invention(aplayer, index) == TECH_KNOWN)
+          && tech_is_available(pplayer, index)) {
        ai_diplomacy_suggest(aplayer, pplayer, CLAUSE_ADVANCE, index);
       } else if ((get_invention(pplayer, index) == TECH_KNOWN)
-          && (get_invention(aplayer, index) != TECH_KNOWN)) {
+          && (get_invention(aplayer, index) != TECH_KNOWN)
+          && tech_is_available(aplayer, index)) {
         ai_diplomacy_suggest(pplayer, aplayer, CLAUSE_ADVANCE, index);
       }
     }
