@@ -42,9 +42,10 @@ Freeciv - Copyright (C) 2004 - The Freeciv Project
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
-#include "rand.h"
 #include "registry.h"
+#include "shared.h"
 #include "support.h"
+
 #include "civclient.h"
 #include "climisc.h"
 #include "clinet.h"
@@ -66,7 +67,7 @@ HANDLE loghandle = INVALID_HANDLE_VALUE;
 static pid_t server_pid = - 1;
 #endif
 
-char player_name[MAX_LEN_NAME];
+char leader_name[MAX_LEN_NAME];
 char *current_filename = NULL;
 
 static char challenge_fullname[MAX_LEN_PATH];
@@ -168,7 +169,7 @@ void client_kill_server(bool force)
       }
       server_process = INVALID_HANDLE_VALUE;
       loghandle = INVALID_HANDLE_VALUE;
-#else
+#elif HAVE_WORKING_FORK
       kill(server_pid, SIGTERM);
       waitpid(server_pid, NULL, WUNTRACED);
       server_pid = -1;
@@ -178,14 +179,6 @@ void client_kill_server(bool force)
   client_has_hack = FALSE;
 }   
 
-/**************************************************************************
- This is called at program exit.
-**************************************************************************/
-static void server_shutdown(void)
-{
-  client_kill_server(TRUE);
-}
-                                                                               
 /**************************************************************** 
 forks a server if it can. returns FALSE is we find we couldn't start
 the server.
@@ -203,7 +196,8 @@ bool client_start_server(void)
 # ifdef WIN32_NATIVE
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
-  
+
+  char savesdir[MAX_LEN_PATH];
   char options[512];
   char cmdline1[512];
   char cmdline2[512];
@@ -211,17 +205,11 @@ bool client_start_server(void)
   char logcmdline[512];
   char scriptcmdline[512];
 # endif
-  static bool initialized = FALSE;
 
   /* only one server (forked from this client) shall be running at a time */
   /* This also resets client_has_hack. */
   client_kill_server(TRUE);
   
-  if (!initialized) {
-    atexit(server_shutdown);
-    initialized = TRUE;
-  }
-
   append_output_window(_("Starting server..."));
 
   /* find a free port */ 
@@ -264,9 +252,9 @@ bool client_start_server(void)
     fclose(stdout);
     fclose(stderr);
 
-    /* include the port to avoid duplication */
+    /* FIXME: include the port to avoid duplication? */
     if (logfile) {
-      fd = open(logfile, O_WRONLY | O_CREAT);
+      fd = open(logfile, O_WRONLY | O_CREAT | O_APPEND, 0644);
 
       if (fd != 1) {
         dup2(fd, 1);
@@ -324,8 +312,10 @@ bool client_start_server(void)
 		scriptfile);
   }
 
-  my_snprintf(options, sizeof(options), "-p %d -q 1 -e%s%s",
-	      internal_server_port, logcmdline, scriptcmdline);
+  interpret_tilde(savesdir, sizeof(savesdir), "~/.freeciv/saves");
+
+  my_snprintf(options, sizeof(options), "-p %d -q 1 -e%s%s --saves \"%s\"",
+	      internal_server_port, logcmdline, scriptcmdline, savesdir);
   my_snprintf(cmdline1, sizeof(cmdline1), "./ser %s", options);
   my_snprintf(cmdline2, sizeof(cmdline2), "./server/civserver %s", options);
   my_snprintf(cmdline3, sizeof(cmdline3), "civserver %s", options);
@@ -384,9 +374,10 @@ bool client_start_server(void)
    * get an iso-map and for a classic tileset you get a classic map.  In
    * both cases the map wraps in the X direction by default.
    *
-   * This works with hex maps too now.  A hex map always has is_isometric
-   * set.  An iso-hex map has hex_height != 0, while a non-iso hex map
-   * has hex_width != 0.
+   * This works with hex maps too now.  A hex map always has
+   * tileset_is_isometric(tileset) return TRUE.  An iso-hex map has
+   * tileset_hex_height(tileset) != 0, while a non-iso hex map
+   * has tileset_hex_width(tileset) != 0.
    *
    * Setting the option here is a bit of a hack, but so long as the client
    * has sufficient permissions to do so (it doesn't have HACK access yet) it
@@ -394,29 +385,16 @@ bool client_start_server(void)
    * set but then overwritten during the load. */
   my_snprintf(buf, sizeof(buf), "/set topology %d",
 	      (TF_WRAPX
-	       | ((is_isometric && hex_height == 0) ? TF_ISO : 0)
-	       | ((hex_width != 0 || hex_height != 0) ? TF_HEX : 0)));
+	       | ((tileset_is_isometric(tileset)
+		   && tileset_hex_height(tileset) == 0) ? TF_ISO : 0)
+	       | ((tileset_hex_width(tileset) != 0
+		   || tileset_hex_height(tileset) != 0) ? TF_HEX : 0)));
   send_chat(buf);
 
   return TRUE;
 #endif
 }
 
-/*************************************************************************
-  generate a random string.
-*************************************************************************/
-static void randomize_string(char *str, size_t n)
-{
-  const char chars[] =
-    "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
-  int i;
-
-  for (i = 0; i < n - 1; i++) {
-    str[i] = chars[myrand(sizeof(chars) - 1)];
-  }
-  str[i] = '\0';
-}
-
 /*************************************************************************
   returns TRUE if a filename is safe (i.e. doesn't have path components).
 *************************************************************************/
@@ -443,53 +421,49 @@ and read the string, then the client is given hack access.
 *****************************************************************/ 
 void send_client_wants_hack(const char *filename)
 {
-  if (has_capability("new_hack", aconnection.capability)) {
-    if (filename[0] != '\0') {
-      struct packet_single_want_hack_req req;
-      struct section_file file;
+  if (filename[0] != '\0') {
+    struct packet_single_want_hack_req req;
+    struct section_file file;
 
-      if (!is_filename_safe(filename)) {
-	return;
-      }
-
-      /* get the full filename path */
-      interpret_tilde(challenge_fullname, sizeof(challenge_fullname),
-	  "~/.freeciv/");
-      make_dir(challenge_fullname);
-
-      sz_strlcat(challenge_fullname, filename);
-
-      /* generate an authentication token */ 
-      randomize_string(req.token, sizeof(req.token));
-
-      section_file_init(&file);
-      secfile_insert_str(&file, req.token, "challenge.token");
-      if (!section_file_save(&file, challenge_fullname, 0)) {
-	freelog(LOG_ERROR, "Couldn't write token to temporary file: %s",
-	    challenge_fullname);
-      }
-      section_file_free(&file);
-
-      /* tell the server what we put into the file */ 
-      send_packet_single_want_hack_req(&aconnection, &req);
+    if (!is_filename_safe(filename)) {
+      return;
     }
+
+    /* get the full filename path */
+    interpret_tilde(challenge_fullname, sizeof(challenge_fullname),
+		    "~/.freeciv/");
+    make_dir(challenge_fullname);
+
+    sz_strlcat(challenge_fullname, filename);
+
+    /* generate an authentication token */ 
+    randomize_base64url_string(req.token, sizeof(req.token));
+
+    section_file_init(&file);
+    secfile_insert_str(&file, req.token, "challenge.token");
+    if (!section_file_save(&file, challenge_fullname, 0)) {
+      freelog(LOG_ERROR, "Couldn't write token to temporary file: %s",
+	      challenge_fullname);
+    }
+    section_file_free(&file);
+
+    /* tell the server what we put into the file */ 
+    send_packet_single_want_hack_req(&aconnection, &req);
   }
 }
 
 /**************************************************************** 
-handle response (by the server) if the client has got hack or not.
+  client has hack (or not).
 *****************************************************************/ 
 void handle_single_want_hack_reply(bool you_have_hack)
 {
-  if (has_capability("new_hack", aconnection.capability)) {
-    /* remove challenge file */
-    if (challenge_fullname[0] != '\0') {
-      if (remove(challenge_fullname) == -1) {
-	freelog(LOG_ERROR, "Couldn't remove temporary file: %s",
-	    challenge_fullname);
-      }
-      challenge_fullname[0] = '\0';
+  /* remove challenge file */
+  if (challenge_fullname[0] != '\0') {
+    if (remove(challenge_fullname) == -1) {
+      freelog(LOG_ERROR, "Couldn't remove temporary file: %s",
+	      challenge_fullname);
     }
+    challenge_fullname[0] = '\0';
   }
 
   if (you_have_hack) {
@@ -514,7 +488,7 @@ void send_start_saved_game(void)
   send_chat("/set timeout 0");
   send_chat("/set autotoggle 1");
   my_snprintf(buf, sizeof(buf), "/take \"%s\" \"%s\"",
-      	      user_name, player_name);
+      	      user_name, leader_name);
   send_chat(buf);
   send_chat("/start");
 }
@@ -534,3 +508,43 @@ void send_save_game(char *filename)
 
   send_chat(message);
 }
+
+/**************************************************************************
+  Handle the list of rulesets sent by the server.
+**************************************************************************/
+void handle_ruleset_choices(struct packet_ruleset_choices *packet)
+{
+  char *rulesets[packet->ruleset_count];
+  int i;
+  size_t suf_len = strlen(RULESET_SUFFIX);
+
+  for (i = 0; i < packet->ruleset_count; i++) {
+    size_t len = strlen(packet->rulesets[i]);
+
+    rulesets[i] = mystrdup(packet->rulesets[i]);
+
+    if (len > suf_len
+	&& strcmp(rulesets[i] + len - suf_len, RULESET_SUFFIX) == 0) {
+      rulesets[i][len - suf_len] = '\0';
+    }
+  }
+  gui_set_rulesets(packet->ruleset_count, rulesets);
+
+  for (i = 0; i < packet->ruleset_count; i++) {
+    free(rulesets[i]);
+  }
+}
+
+/**************************************************************************
+  Called by the GUI code when the user sets the ruleset.  The ruleset
+  passed in here should match one of the strings given to gui_set_rulesets.
+**************************************************************************/
+void set_ruleset(const char *ruleset)
+{
+  char buf[4096];
+
+  my_snprintf(buf, sizeof(buf), "/read %s%s",
+	      ruleset, RULESET_SUFFIX);
+  freelog(LOG_DEBUG, "Executing '%s'", buf);
+  send_chat(buf);
+}
