@@ -1,4 +1,4 @@
-/********************************************************************** 
+/********************7************************************************** 
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 
 #include "city.h"
 #include "citydlg_common.h"
+#include "civclient.h"
 #include "fcintl.h"
 #include "game.h"
 #include "gui_main.h"
@@ -39,9 +40,10 @@
 #include "options.h"
 #include "tilespec.h"
 
-#include "wldlg.h"
+#include "canvas.h"
 #include "citydlg.h"
-#include "civclient.h"
+#include "graphics.h"
+#include "wldlg.h"
 
 static GtkWidget *worklists_shell;
 static GtkWidget *worklists_list;
@@ -59,7 +61,8 @@ static int max_unit_height = -1, max_unit_width = -1;
 
 static void popup_worklist(struct worklist *pwl);
 static void popdown_worklist(struct worklist *pwl);
-
+static void dst_row_callback(GtkTreeView *view, GtkTreePath *path,
+			     GtkTreeViewColumn *col, gpointer data);
 
 /****************************************************************
 ...
@@ -79,10 +82,10 @@ static void update_max_unit_size(void)
   max_unit_width = 0;
 
   unit_type_iterate(i) {
-    struct unit_type *type = get_unit_type(i);
     int x1, x2, y1, y2;
+    struct sprite *sprite = get_unittype_sprite(tileset, i);
 
-    sprite_get_bounding_box(type->sprite, &x1, &y1, &x2, &y2);
+    sprite_get_bounding_box(sprite, &x1, &y1, &x2, &y2);
     max_unit_width = MAX(max_unit_width, x2 - x1);
     max_unit_height = MAX(max_unit_height, y2 - y1);
   } unit_type_iterate_end;
@@ -114,7 +117,11 @@ void update_worklist_report_dialog(void)
   plr = game.player_ptr;
 
   gtk_list_store_clear(worklists_store);
-  
+
+  if (!game.player_ptr) {
+    return;
+  }
+
   for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
     if (plr->worklists[i].is_valid) {
       gtk_list_store_append(worklists_store, &it);
@@ -140,6 +147,10 @@ static void worklists_response(GtkWidget *w, gint response)
 
   plr = game.player_ptr;
 
+  if (!game.player_ptr) {
+    return;
+  }
+
   for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
     if (!plr->worklists[i].is_valid) {
       break;
@@ -206,13 +217,12 @@ static void cell_edited(GtkCellRendererText *cell,
   GtkTreePath *path;
   GtkTreeIter it;
   int pos;
-  struct player *plr;
+  struct player *plr = game.player_ptr;
 
   path = gtk_tree_path_new_from_string(spath);
   gtk_tree_model_get_iter(GTK_TREE_MODEL(worklists_store), &it, path);
   
   gtk_tree_model_get(GTK_TREE_MODEL(worklists_store), &it, 1, &pos, -1);
-  plr = game.player_ptr;
 
   sz_strlcpy(plr->worklists[pos].name, text);
   gtk_list_store_set(worklists_store, &it, 0, text, -1);
@@ -322,7 +332,7 @@ struct worklist_data {
   GtkTreeViewColumn *src_col, *dst_col;
 
   GtkWidget *add_cmd, *change_cmd, *help_cmd;
-  GtkWidget *up_cmd, *down_cmd, *prepend_cmd, *append_cmd;
+  GtkWidget *up_cmd, *down_cmd, *prepend_cmd, *append_cmd, *remove_cmd;
 
   bool future;
 };
@@ -470,6 +480,10 @@ static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
   gint pos;
   struct worklist *pwl;
 
+  if (!game.player_ptr) {
+    return;
+  }
+
   plr = game.player_ptr;
   pos = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(item), "pos"));
 
@@ -478,15 +492,11 @@ static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
   if (pwl->is_valid) {
     int i;
 
-    for (i = 0; i < MAX_LEN_WORKLIST; i++) {
+    for (i = 0; i < worklist_length(pwl); i++) {
       GtkTreeIter it;
       cid cid;
 
-      if (pwl->wlefs[i] == WEF_END) {
-	break;
-      }
-
-      cid = cid_encode(pwl->wlefs[i] == WEF_UNIT, pwl->wlids[i]);
+      cid = cid_encode(pwl->entries[i]);
 
       gtk_list_store_append(ptr->dst, &it);
       gtk_list_store_set(ptr->dst, &it, 0, (gint) cid, -1);
@@ -509,6 +519,10 @@ static void popup_add_menu(GtkMenuShell *menu, gpointer data)
 			(GtkCallback) gtk_widget_destroy, NULL);
   plr = game.player_ptr;
 
+  if (!game.player_ptr) {
+    return;
+  }
+
   for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
     if (plr->worklists[i].is_valid) {
       item = gtk_menu_item_new_with_label(plr->worklists[i].name);
@@ -549,19 +563,20 @@ static void help_callback(GtkWidget *w, gpointer data)
 
   if (gtk_tree_selection_get_selected(selection, &model, &it)) {
     gint cid;
-    int id;
-    bool is_unit;
+    struct city_production target;
 
     gtk_tree_model_get(model, &it, 0, &cid, -1);
-    is_unit = cid_is_unit(cid);
-    id = cid_id(cid);
+    target = cid_decode(cid);
 
-    if (is_unit) {
-      popup_help_dialog_typed(get_unit_type(id)->name, HELP_UNIT);
-    } else if (is_wonder(id)) {
-      popup_help_dialog_typed(get_improvement_name(id), HELP_WONDER);
+    if (target.is_unit) {
+      popup_help_dialog_typed(utype_name_translation(utype_by_number(target.value)),
+			      HELP_UNIT);
+    } else if (is_great_wonder(target.value)) {
+      popup_help_dialog_typed(improvement_name_translation(target.value),
+			      HELP_WONDER);
     } else {
-      popup_help_dialog_typed(get_improvement_name(id), HELP_IMPROVEMENT);
+      popup_help_dialog_typed(improvement_name_translation(target.value),
+			      HELP_IMPROVEMENT);
     }
   } else {
     popup_help_dialog_string(HELP_WORKLIST_EDITOR_ITEM);
@@ -586,7 +601,7 @@ static void change_callback(GtkWidget *w, gpointer data)
 
     gtk_tree_model_get(model, &it, 0, &cid, -1);
 
-    city_change_production(ptr->pcity, cid_is_unit(cid), cid_id(cid));
+    city_change_production(ptr->pcity, cid_production(cid));
   }
 }
 
@@ -635,6 +650,21 @@ static void queue_bubble_up(struct worklist_data *ptr)
   gtk_tree_path_free(path);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void queue_remove(struct worklist_data *ptr)
+{
+  GtkTreePath *path;
+  GtkTreeViewColumn *col;
+
+  gtk_tree_view_get_cursor(GTK_TREE_VIEW(ptr->dst_view), &path, &col);
+  if (path) {
+    dst_row_callback(GTK_TREE_VIEW(ptr->dst_view), path, col, ptr);
+    gtk_tree_path_free(path);
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -868,7 +898,8 @@ static void src_selection_callback(GtkTreeSelection *selection, gpointer data)
   /* update widget sensitivity. */
   if (gtk_tree_selection_get_selected(selection, NULL, NULL)) {
     if (can_client_issue_orders()
-	&& ptr->pcity && city_owner(ptr->pcity) == game.player_ptr) {
+      && (!ptr->pcity || city_owner(ptr->pcity) == game.player_ptr)) {
+      /* if ptr->pcity is NULL, this is a global worklist */
       gtk_widget_set_sensitive(ptr->change_cmd, TRUE);
       gtk_widget_set_sensitive(ptr->prepend_cmd, TRUE);
       gtk_widget_set_sensitive(ptr->append_cmd, TRUE);
@@ -897,11 +928,25 @@ static void dst_selection_callback(GtkTreeSelection *selection, gpointer data)
 
   /* update widget sensitivity. */
   if (gtk_tree_selection_count_selected_rows(selection) > 0) {
+    int num_rows = 0;
+    GtkTreeIter it;
+    
     gtk_widget_set_sensitive(ptr->up_cmd, TRUE);
     gtk_widget_set_sensitive(ptr->down_cmd, TRUE);
+    if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ptr->dst), &it)) {
+      do {
+	num_rows++;
+      } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(ptr->dst), &it));
+    }
+    if (num_rows > 1) {
+      gtk_widget_set_sensitive(ptr->remove_cmd, TRUE);      
+    } else {
+      gtk_widget_set_sensitive(ptr->remove_cmd, FALSE);
+    }
   } else {
     gtk_widget_set_sensitive(ptr->up_cmd, FALSE);
     gtk_widget_set_sensitive(ptr->down_cmd, FALSE);
+    gtk_widget_set_sensitive(ptr->remove_cmd, FALSE);
   }
 }
 
@@ -922,17 +967,16 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 			     GtkTreeModel *model, GtkTreeIter *it,
 			     gpointer data)
 {
-  gint cid, id;
-  bool is_unit;
+  gint cid;
+  struct city_production target;
 
   gtk_tree_model_get(model, it, 0, &cid, -1);
-  is_unit = cid_is_unit(cid);
-  id = cid_id(cid);
+  target = cid_production(cid);
 
   if (GTK_IS_CELL_RENDERER_PIXBUF(rend)) {
     GdkPixbuf *pix;
 
-    if (is_unit) {
+    if (target.is_unit) {
       struct canvas store;
 
       pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
@@ -940,18 +984,18 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 
       store.type = CANVAS_PIXBUF;
       store.v.pixbuf = pix;
-      create_overlay_unit(&store, id);
+      create_overlay_unit(&store, utype_by_number(target.value));
 
       g_object_set(rend, "pixbuf", pix, NULL);
       g_object_unref(pix);
     } else {
-      struct impr_type *impr = get_improvement_type(id);
+      struct sprite *sprite = get_building_sprite(tileset, target.value);
 
-      pix = sprite_get_pixbuf(impr->sprite);
+      pix = sprite_get_pixbuf(sprite);
       g_object_set(rend, "pixbuf", pix, NULL);
     }
   } else {
-    struct city **pcity;
+    struct city **pcity = data;
     struct player *plr;
     gint column;
     char *row[4];
@@ -959,20 +1003,21 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
     int   i;
     gboolean useless;
 
-    pcity = (struct city **) data;
-
     for (i = 0; i < ARRAY_SIZE(row); i++) {
       row[i] = buf[i];
     }
     column = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(rend), "column"));
 
-    get_city_dialog_production_row(row, sizeof(buf[0]), id, is_unit, *pcity);
+    get_city_dialog_production_row(row, sizeof(buf[0]), target, *pcity);
     g_object_set(rend, "text", row[column], NULL);
 
-    if (!is_unit && *pcity) {
+    if (!target.is_unit && *pcity) {
       plr = city_owner(*pcity);
-      useless = improvement_obsolete(plr, id)
-	|| improvement_redundant(plr, *pcity, id, FALSE);
+      useless = improvement_obsolete(plr, target.value)
+        || (! impr_provides_buildable_units(plr, target.value)
+            && is_building_replaced(*pcity, target.value, RPT_CERTAIN));
+      /* Mark building redundant if we are really certain that there is
+       * no use for it. */
       g_object_set(rend, "strikethrough", useless, NULL);
     } else {
       g_object_set(rend, "strikethrough", FALSE, NULL);
@@ -991,46 +1036,51 @@ static void populate_view(GtkTreeView *view, struct city **ppcity,
 
   static bool titles_done;
   gint i;
-
+  GtkCellRenderer *rend;
+  GtkTreeViewColumn *col;
 
   intl_slist(ARRAY_SIZE(titles), titles, &titles_done);
 
-  for (i = 0; i < ARRAY_SIZE(titles); i++) {
-    GtkCellRenderer *rend;
-    GtkTreeViewColumn *col;
+  /* Case i == 0 taken out of the loop to workaround gcc-4.2.1 bug
+   * http://gcc.gnu.org/PR33381
+   * Some values would 'stick' from i == 0 round. */
+  i = 0;
 
-    if (i == 0) {
-      rend = gtk_cell_renderer_pixbuf_new();
+  rend = gtk_cell_renderer_pixbuf_new();
 
-      gtk_tree_view_insert_column_with_data_func(view,
-	  i, titles[i], rend, cell_render_func, ppcity, NULL);
-      col = gtk_tree_view_get_column(view, i);
+  gtk_tree_view_insert_column_with_data_func(view,
+                   i, titles[i], rend, cell_render_func, ppcity, NULL);
+  col = gtk_tree_view_get_column(view, i);
 
-      if (show_task_icons) {
-	if (max_unit_width == -1 || max_unit_height == -1) {
-	  update_max_unit_size();
-	}
-      } else {
-	g_object_set(col, "visible", FALSE, NULL);
-      }
-    } else {
-      gint pos = i-1;
+  if (show_task_icons) {
+    if (max_unit_width == -1 || max_unit_height == -1) {
+      update_max_unit_size();
+    }
+  } else {
+    g_object_set(col, "visible", FALSE, NULL);
+  }
+  if (show_task_icons) {
+    g_object_set(rend, "height", max_unit_height, NULL);
+  }
 
-      rend = gtk_cell_renderer_text_new();
-      g_object_set_data(G_OBJECT(rend), "column", GINT_TO_POINTER(pos));
+  for (i = 1; i < ARRAY_SIZE(titles); i++) {
 
-      gtk_tree_view_insert_column_with_data_func(view,
-	  i, titles[i], rend, cell_render_func, ppcity, NULL); 
-      col = gtk_tree_view_get_column(view, i);
+    gint pos = i-1;
 
-      if (pos >= 2) {
-	g_object_set(G_OBJECT(rend), "xalign", 1.0, NULL);
-	gtk_tree_view_column_set_alignment(col, 1.0);
-      }
+    rend = gtk_cell_renderer_text_new();
+    g_object_set_data(G_OBJECT(rend), "column", GINT_TO_POINTER(pos));
 
-      if (pos == 3) {
-	*pcol = col;
-      }
+    gtk_tree_view_insert_column_with_data_func(view,
+	i, titles[i], rend, cell_render_func, ppcity, NULL); 
+    col = gtk_tree_view_get_column(view, i);
+
+    if (pos >= 2) {
+      g_object_set(G_OBJECT(rend), "xalign", 1.0, NULL);
+      gtk_tree_view_column_set_alignment(col, 1.0);
+    }
+
+    if (pos == 3) {
+      *pcol = col;
     }
     if (show_task_icons) {
       g_object_set(rend, "height", max_unit_height, NULL);
@@ -1109,7 +1159,7 @@ GtkWidget *create_worklist(void)
   g_signal_connect(check, "toggled", G_CALLBACK(future_callback), ptr);
 
 
-  table2 = gtk_table_new(4, 1, FALSE);
+  table2 = gtk_table_new(5, 1, FALSE);
   gtk_table_attach(GTK_TABLE(table), table2, 2, 3, 1, 2,
 		   GTK_FILL, GTK_FILL, 0, 0);
 
@@ -1159,6 +1209,17 @@ GtkWidget *create_worklist(void)
 			   G_CALLBACK(queue_append), ptr);
   gtk_widget_set_sensitive(ptr->append_cmd, FALSE);
 
+  button = gtk_button_new();
+  ptr->remove_cmd = button;
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  gtk_table_attach(GTK_TABLE(table2), button, 0, 1, 4, 5,
+      0, GTK_EXPAND|GTK_FILL, 0, 24);
+  
+  arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_IN);
+  gtk_container_add(GTK_CONTAINER(button), arrow);
+  g_signal_connect_swapped(button, "clicked",
+                           G_CALLBACK(queue_remove), ptr);
+  gtk_widget_set_sensitive(ptr->remove_cmd, FALSE);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
@@ -1296,9 +1357,9 @@ void refresh_worklist(GtkWidget *editor)
   struct worklist_data *ptr;
   struct worklist *pwl, queue;
 
-  cid cids[U_LAST + B_LAST];
-  int i, cids_used;
-  struct item items[U_LAST + B_LAST];
+  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  int i, targets_used;
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
 
   bool selected;
   gint id;
@@ -1325,15 +1386,15 @@ void refresh_worklist(GtkWidget *editor)
   }
   gtk_list_store_clear(ptr->src);
 
-  cids_used = collect_cids4(cids, ptr->pcity, ptr->future);
-  name_and_sort_items(cids, cids_used, items, FALSE, ptr->pcity);
+  targets_used = collect_eventually_buildable_targets(targets, ptr->pcity, ptr->future);
+  name_and_sort_items(targets, targets_used, items, FALSE, ptr->pcity);
 
   path = NULL;
-  for (i = 0; i < cids_used; i++) {
+  for (i = 0; i < targets_used; i++) {
     gtk_list_store_append(ptr->src, &it);
-    gtk_list_store_set(ptr->src, &it, 0, (gint) items[i].cid, -1);
+    gtk_list_store_set(ptr->src, &it, 0, (gint) cid_encode(items[i].item), -1);
 
-    if (selected && items[i].cid == id) {
+    if (selected && cid_encode(items[i].item) == id) {
       path = gtk_tree_model_get_path(GTK_TREE_MODEL(ptr->src), &it);
     }
   }
@@ -1354,20 +1415,14 @@ void refresh_worklist(GtkWidget *editor)
     copy_worklist(&queue, pwl);
   }
 
-  for (i = 0; i < MAX_LEN_WORKLIST; i++) {
-    cid cid;
-
-    if (queue.wlefs[i] == WEF_END) {
-      break;
-    }
-
-    cid = cid_encode(queue.wlefs[i] == WEF_UNIT, queue.wlids[i]);
+  for (i = 0; i < worklist_length(&queue); i++) {
+    struct city_production target = queue.entries[i];
 
     if (!exists) {
       gtk_list_store_append(ptr->dst, &it);
     }
 
-    gtk_list_store_set(ptr->dst, &it, 0, (gint) cid, -1);
+    gtk_list_store_set(ptr->dst, &it, 0, (gint) cid_encode(target), -1);
 
     if (exists) {
       exists = gtk_tree_model_iter_next(model, &it);
@@ -1435,8 +1490,8 @@ static void commit_worklist(struct worklist_data *ptr)
       }
 
       gtk_tree_model_get(model, &it, 0, &cid, -1);
-      queue.wlefs[i] = cid_is_unit(cid) ? WEF_UNIT : WEF_IMPR;
-      queue.wlids[i] = cid_id(cid);
+
+      worklist_append(&queue, cid_production(cid));
 
       i++;
     } while (gtk_tree_model_iter_next(model, &it));
