@@ -448,22 +448,21 @@ void handle_city_info(struct packet_city_info *packet)
   } else {
     name_changed = (strcmp(city_name(pcity), packet->name) != 0);
 
-    /* Check if city desciptions should be updated */
-    if (draw_city_names && name_changed) {
-      update_descriptions = TRUE;
-    } else if (DRAW_CITY_PRODUCTIONS
-	       && (pcity->production.is_unit != packet->production_is_unit
-		   || pcity->production.value != packet->production_value
-		   || pcity->surplus[O_SHIELD] != packet->surplus[O_SHIELD]
-		   || pcity->shield_stock != packet->shield_stock)) {
-      update_descriptions = TRUE;
-    } else if (draw_city_names && DRAW_CITY_GROWTH &&
-	       (pcity->food_stock != packet->food_stock ||
-		pcity->surplus[O_FOOD] != packet->surplus[O_FOOD])) {
-      /* If either the food stock or surplus have changed, the time-to-grow
-	 is likely to have changed as well. */
-      update_descriptions = TRUE;
-    }
+    /* Descriptions should probably be updated if the
+     * city name, production or time-to-grow changes.
+     * Note that if either the food stock or surplus
+     * have changed, the time-to-grow is likely to
+     * have changed as well. */
+    update_descriptions = (draw_city_names && name_changed)
+      || (draw_city_productions
+          && (pcity->production.is_unit != packet->production_is_unit
+              || pcity->production.value != packet->production_value
+              || pcity->surplus[O_SHIELD] != packet->surplus[O_SHIELD]
+              || pcity->shield_stock != packet->shield_stock))
+      || (draw_city_growth
+          && (pcity->food_stock != packet->food_stock
+              || pcity->surplus[O_FOOD] != packet->surplus[O_FOOD]));
+
     assert(pcity->id == packet->id);
   }
   
@@ -1531,6 +1530,11 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->is_male=pinfo->is_male;
   team_add_player(pplayer, team_get_by_id(pinfo->team));
   pplayer->score.game = pinfo->score;
+  if (has_capability("CreatedName", aconnection.capability)) {
+    pplayer->was_created = pinfo->was_created;
+  } else {
+    pplayer->was_created = FALSE;
+  }
 
   pplayer->economic.gold=pinfo->gold;
   pplayer->economic.tax=pinfo->tax;
@@ -2124,20 +2128,8 @@ void handle_tile_info(struct packet_tile_info *packet)
     unit_list_unlink_all(ptile->units);
   }
 
-  /* update continents */
-  if (ptile->continent != packet->continent && ptile->continent != 0
-      && packet->continent > 0) {
-    /* We're renumbering continents, somebody did a transform.
-     * But we don't care about renumbering oceans since 
-     * num_oceans is not kept at the client. */
-    map.num_continents = 0;
-  }
-
   ptile->continent = packet->continent;
-
-  if (ptile->continent > map.num_continents) {
-    map.num_continents = ptile->continent;
-  }
+  map.num_continents = MAX(ptile->continent, map.num_continents);
 
   if (known_changed || tile_changed) {
     /* 
@@ -2174,6 +2166,12 @@ void handle_tile_info(struct packet_tile_info *packet)
 **************************************************************************/
 void handle_player_remove(int player_id)
 {
+  if (player_id < 0 || player_id >= MAX_NUM_PLAYERS) {
+    freelog(LOG_ERROR, "Illegal player_remove packet from server (%d)",
+            player_id);
+    return;
+  }
+
   client_remove_player(player_id);
   update_conn_list_dialog();
 }
@@ -2360,7 +2358,15 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
     } impr_type_iterate_end;
   }
 #endif
-  
+
+  b->allows_units = FALSE;
+  unit_type_iterate(ut) {
+    if (ut->impr_requirement == p->id) {
+      b->allows_units = TRUE;
+      break;
+    }
+  } unit_type_iterate_end;
+
   tileset_setup_impr_type(tileset, p->id);
 }
 
