@@ -25,39 +25,37 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "events.h"
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
 #include "log.h"
-#include "map.h"
-#include "nation.h"
 #include "rand.h"
 #include "support.h"
+
+#include "effects.h"
+#include "events.h"
+#include "game.h"
+#include "government.h"
+#include "map.h"
+#include "movement.h"
+#include "nation.h"
 #include "tech.h"
 #include "terrain.h"
+#include "unitlist.h"
 
+#include "barbarian.h"
 #include "gamehand.h"
-#include "gamelog.h"
 #include "maphand.h"
 #include "plrhand.h"
 #include "srv_main.h"
 #include "stdinhand.h"
+#include "techtools.h"
 #include "unithand.h"
 #include "unittools.h"
 
 #include "aidata.h"
 #include "aitools.h"
 
-#include "barbarian.h"
-
-/*
- IDEAS:
- 1. Unrest factors configurable via rulesets (distance and gov factor)
- 2. Separate nations for Sea Raiders and Land Barbarians
-
- - are these good ideas ??????
-*/
+#define BARBARIAN_INITIAL_VISION_RADIUS 3
+#define BARBARIAN_INITIAL_VISION_RADIUS_SQ 9
 
 /**************************************************************************
   Is player a land barbarian?
@@ -75,6 +73,30 @@ static bool is_sea_barbarian(struct player *pplayer)
   return (pplayer->ai.barbarian_type == SEA_BARBARIAN);
 }
 
+/****************************************************************************
+  Return an available barbarian nation.  This simply returns the first
+  available nation, or the first nation already in use by another barbarian
+  player.
+****************************************************************************/
+struct nation_type *pick_barbarian_nation(void)
+{
+  nations_iterate(pnation) {
+    if (is_nation_barbarian(pnation) && !pnation->player) {
+      return pnation;
+    }
+  } nations_iterate_end;
+
+  players_iterate(pplayer) {
+    if (is_barbarian(pplayer)) {
+      assert(is_nation_barbarian(nation_of_player(pplayer)));
+      return nation_of_player(pplayer);
+    }
+  } players_iterate_end;
+
+  assert(0);
+  return NO_NATION_SELECTED;
+}
+
 /**************************************************************************
   Creates the land/sea barbarian player and inits some stuff. If 
   barbarian player already exists, return player pointer. If barbarians 
@@ -84,8 +106,9 @@ static bool is_sea_barbarian(struct player *pplayer)
 **************************************************************************/
 static struct player *create_barbarian_player(bool land)
 {
-  int newplayer = game.nplayers;
+  int newplayer = game.info.nplayers;
   struct player *barbarians;
+  struct nation_type *nation;
 
   players_iterate(barbarians) {
     if ((land && is_land_barbarian(barbarians))
@@ -94,7 +117,7 @@ static struct player *create_barbarian_player(bool land)
         barbarians->economic.gold = 0;
         barbarians->is_alive = TRUE;
         barbarians->is_dying = FALSE;
-        pick_ai_player_name(game.nation_count - 1, barbarians->name);
+        pick_random_player_name(nation_of_player(barbarians), barbarians->name);
 	sz_strlcpy(barbarians->username, ANON_USER_NAME);
         /* I need to make them to forget the map, I think */
 	whole_map_iterate(ptile) {
@@ -110,28 +133,29 @@ static struct player *create_barbarian_player(bool land)
     die("Too many players in server/barbarian.c");
   }
 
+  nation = pick_barbarian_nation();
+
   barbarians = &game.players[newplayer];
 
   /* make a new player */
 
-  server_player_init(barbarians, TRUE);
+  server_player_init(barbarians, TRUE, TRUE);
 
-  barbarians->nation = game.nation_count - 1;
-  pick_ai_player_name(game.nation_count - 1, barbarians->name);
+  player_set_nation(barbarians, nation);
+  pick_random_player_name(nation, barbarians->name);
 
-  game.nplayers++;
-  game.nbarbarians++;
-  game.max_players = game.nplayers;
+  game.info.nplayers++;
+  game.info.nbarbarians++;
+  game.info.max_players = game.info.nplayers;
 
   sz_strlcpy(barbarians->username, ANON_USER_NAME);
   barbarians->is_connected = FALSE;
-  barbarians->government = game.default_government;
-  barbarians->target_government = game.default_government;
+  barbarians->government = nation->init_government;
   assert(barbarians->revolution_finishes < 0);
   barbarians->capital = FALSE;
   barbarians->economic.gold = 100;
 
-  barbarians->turn_done = TRUE;
+  barbarians->phase_done = TRUE;
 
   /* Do the ai */
   barbarians->ai.control = TRUE;
@@ -140,24 +164,26 @@ static struct player *create_barbarian_player(bool land)
   } else {
     barbarians->ai.barbarian_type = SEA_BARBARIAN;
   }
-  set_ai_level_directer(barbarians, game.skill_level);
-  init_tech(barbarians);
+  set_ai_level_directer(barbarians, game.info.skill_level);
+  init_tech(barbarians, TRUE);
   give_initial_techs(barbarians);
 
   /* Ensure that we are at war with everyone else */
   players_iterate(pplayer) {
     if (pplayer != barbarians) {
-      pplayer->diplstates[barbarians->player_no].type = DS_WAR;
-      barbarians->diplstates[pplayer->player_no].type = DS_WAR;
+      pplayer->diplstates[player_index(barbarians)].type = DS_WAR;
+      barbarians->diplstates[player_index(pplayer)].type = DS_WAR;
     }
   } players_iterate_end;
 
   freelog(LOG_VERBOSE, "Created barbarian %s, player %d",
-          barbarians->name, barbarians->player_no);
-  notify_player_ex(NULL, NULL, E_UPRISING,
-                   _("Barbarians gain a leader by the name %s.  Dangerous "
-                     "times may lie ahead."), barbarians->name);
-  gamelog(GAMELOG_PLAYER, barbarians);
+          player_name(barbarians),
+          player_number(barbarians));
+  notify_player(NULL, NULL, E_UPRISING,
+                _("%s gain a leader by the name %s. Dangerous "
+                  "times may lie ahead."),
+                nation_plural_for_player(barbarians),
+                player_name(barbarians));
 
   send_game_info(NULL);
   send_player_info(barbarians, NULL);
@@ -170,7 +196,7 @@ static struct player *create_barbarian_player(bool land)
 **************************************************************************/
 static bool is_free_land(struct tile *ptile, struct player *who)
 {
-  return (!is_ocean(map_get_terrain(ptile))
+  return (!is_ocean(tile_get_terrain(ptile))
 	  && !is_non_allied_unit_tile((ptile), who));
 }
 
@@ -179,7 +205,7 @@ static bool is_free_land(struct tile *ptile, struct player *who)
 **************************************************************************/
 static bool is_free_sea(struct tile *ptile, struct player *who)
 {
-  return (is_ocean(map_get_terrain(ptile))
+  return (is_ocean(tile_get_terrain(ptile))
 	  && !is_non_allied_unit_tile((ptile), who));
 }
 
@@ -196,28 +222,37 @@ static bool is_free_sea(struct tile *ptile, struct player *who)
 bool unleash_barbarians(struct tile *ptile)
 {
   struct player *barbarians;
-  int unit, unit_cnt, land_cnt = 0, sea_cnt = 0;
-  int boat;
-  int i, me;
+  int unit_cnt, land_cnt = 0, sea_cnt = 0;
+  int i;
   struct tile *utile = NULL;
   bool alive = TRUE;     /* explorer survived */
 
-  if (game.barbarianrate == 0 || (game.year < game.onsetbarbarian)) {
+  /* FIXME: When there is no L_BARBARIAN unit,
+   *        but L_BARBARIAN_TECH is already available,
+   *        we should unleash those.
+   *        Doesn't affect any ruleset I'm aware of. */
+  if (game.info.barbarianrate == 0
+      || game.info.year < game.info.onsetbarbarian
+      || num_role_units(L_BARBARIAN) == 0) {
     unit_list_iterate_safe((ptile)->units, punit) {
       wipe_unit(punit);
     } unit_list_iterate_safe_end;
     return FALSE;
   }
 
-  unit_cnt = 3 + myrand(4);
-
   barbarians = create_barbarian_player(TRUE);
-  me = barbarians->player_no;
+  if (!barbarians) {
+    return FALSE;
+  }
 
+  unit_cnt = 3 + myrand(4);
   for (i = 0; i < unit_cnt; i++) {
-    unit = find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
-    (void) create_unit(barbarians, ptile, unit, 0, 0, -1);
-    freelog(LOG_DEBUG, "Created barbarian unit %s", unit_types[unit].name);
+    struct unit_type *punittype
+      = find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
+
+    (void) create_unit(barbarians, ptile, punittype, 0, 0, -1);
+    freelog(LOG_DEBUG, "Created barbarian unit %s",
+            utype_rule_name(punittype));
   }
 
   adjc_iterate(ptile, tile1) {
@@ -227,7 +262,7 @@ bool unleash_barbarians(struct tile *ptile)
 
   if (land_cnt >= 3) {           /* enough land, scatter guys around */
     unit_list_iterate((ptile)->units, punit2) {
-      if (punit2->owner == me) {
+      if (unit_owner(punit2) == barbarians) {
         send_unit_info(NULL, punit2);
 	do {
 	  do {
@@ -245,7 +280,7 @@ bool unleash_barbarians(struct tile *ptile)
       /* FIXME: If anyone knows what this code is supposed to do, rewrite
        * this comment to explain it. */
       unit_list_iterate((ptile)->units, punit2) {
-        if (punit2->owner == me) {
+        if (unit_owner(punit2) == barbarians) {
           send_unit_info(NULL, punit2);
           while(TRUE) {
 	    utile = rand_neighbour(ptile);
@@ -257,7 +292,7 @@ bool unleash_barbarians(struct tile *ptile)
               break;
 	    }
 	    if (is_free_sea(utile, barbarians)) {
-              boat = find_a_unit_type(L_BARBARIAN_BOAT, -1);
+              struct unit_type *boat = find_a_unit_type(L_BARBARIAN_BOAT, -1);
 	      (void) create_unit(barbarians, utile, boat, 0, 0, -1);
 	      btile = utile;
 	      break;
@@ -268,7 +303,7 @@ bool unleash_barbarians(struct tile *ptile)
       } unit_list_iterate_end;
     } else {             /* The village is surrounded! Kill the explorer. */
       unit_list_iterate_safe((ptile)->units, punit2) {
-        if (punit2->owner != me) {
+        if (unit_owner(punit2) != barbarians) {
           wipe_unit(punit2);
           alive = FALSE;
         } else {
@@ -280,7 +315,7 @@ bool unleash_barbarians(struct tile *ptile)
 
   /* FIXME: I don't know if this is needed */
   if (utile) {
-    show_area(barbarians, utile, 3);
+    map_show_circle(barbarians, utile, BARBARIAN_INITIAL_VISION_RADIUS_SQ);
   }
 
   return alive;
@@ -292,7 +327,7 @@ bool unleash_barbarians(struct tile *ptile)
 static bool is_near_land(struct tile *tile0)
 {
   square_iterate(tile0, 4, ptile) {
-    if (!is_ocean(map_get_terrain(ptile))) {
+    if (!is_ocean(tile_get_terrain(ptile))) {
       return TRUE;
     }
   } square_iterate_end;
@@ -306,7 +341,7 @@ static bool is_near_land(struct tile *tile0)
 static struct tile *find_empty_tile_nearby(struct tile *ptile)
 {
   square_iterate(ptile, 1, tile1) {
-    if (unit_list_size(&(tile1)->units) == 0) {
+    if (unit_list_size(tile1->units) == 0) {
       return tile1;
     }
   } square_iterate_end;
@@ -335,7 +370,7 @@ static struct tile *find_empty_tile_nearby(struct tile *ptile)
 static void try_summon_barbarians(void)
 {
   struct tile *ptile, *utile;
-  int i, boat, cap, dist, unit;
+  int i, cap, dist;
   int uprise = 1;
   struct city *pc;
   struct player *barbarians, *victim;
@@ -347,7 +382,7 @@ static void try_summon_barbarians(void)
    * gameplay. */
   ptile = rand_map_pos();
 
-  if (terrain_has_flag(map_get_terrain(ptile), TER_NO_BARBS)) {
+  if (terrain_has_flag(tile_get_terrain(ptile), TER_NO_BARBS)) {
     return;
   }
 
@@ -360,8 +395,8 @@ static void try_summon_barbarians(void)
   victim = city_owner(pc);
 
   dist = real_map_distance(ptile, pc->tile);
-  freelog(LOG_DEBUG,"Closest city to %d %d is %s at %d %d which is %d far", 
-          ptile->x, ptile->y, pc->name, pc->tile->x, pc->tile->y, dist);
+  freelog(LOG_DEBUG,"Closest city (to %d,%d) is %s (at %d,%d) distance %d.", 
+          TILE_XY(ptile), city_name(pc), TILE_XY(pc->tile), dist);
   if (dist > MAX_UNREST_DIST || dist < MIN_UNREST_DIST) {
     return;
   }
@@ -369,51 +404,60 @@ static void try_summon_barbarians(void)
   /* I think Sea Raiders can come out of unknown sea territory */
   if (!(utile = find_empty_tile_nearby(ptile))
       || (!map_is_known(utile, victim)
-	  && !is_ocean(map_get_terrain(utile)))
+	  && !is_ocean(tile_get_terrain(utile)))
       || !is_near_land(utile)) {
     return;
   }
 
   /* do not harass small civs - in practice: do not uprise at the beginning */
   if ((int)myrand(UPRISE_CIV_MORE) >
-           (int)city_list_size(&victim->cities) -
-                UPRISE_CIV_SIZE/(game.barbarianrate-1)
-      || myrand(100) > get_gov_pcity(pc)->civil_war) {
+           (int)city_list_size(victim->cities) -
+                UPRISE_CIV_SIZE/(game.info.barbarianrate-1)
+      || myrand(100) > get_player_bonus(victim, EFT_CIVIL_WAR_CHANCE)) {
     return;
   }
   freelog(LOG_DEBUG, "Barbarians are willing to fight");
 
-  if (map_has_special(utile, S_HUT)) {
+  if (tile_has_special(utile, S_HUT)) {
     /* remove the hut in place of uprising */
-    map_clear_special(utile, S_HUT);
+    tile_clear_special(utile, S_HUT);
     update_tile_knowledge(utile);
   }
 
-  if (!is_ocean(map_get_terrain(utile))) {
+  if (!is_ocean(tile_get_terrain(utile))) {
+    int rand_factor = myrand(3);
+
     /* land (disembark) barbarians */
     barbarians = create_barbarian_player(TRUE);
-    if (city_list_size(&victim->cities) > UPRISE_CIV_MOST) {
+    if (city_list_size(victim->cities) > UPRISE_CIV_MOST) {
       uprise = 3;
     }
-    for (i = 0; i < myrand(3) + uprise * game.barbarianrate; i++) {
-      unit = find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
-      (void) create_unit(barbarians, utile, unit, 0, 0, -1);
-      freelog(LOG_DEBUG, "Created barbarian unit %s", unit_types[unit].name);
+    for (i = 0; i < rand_factor + uprise * game.info.barbarianrate; i++) {
+      struct unit_type *punittype
+	= find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
+
+      (void) create_unit(barbarians, utile, punittype, 0, 0, -1);
+      freelog(LOG_DEBUG, "Created barbarian unit %s",
+              utype_rule_name(punittype));
     }
     (void) create_unit(barbarians, utile,
 		       get_role_unit(L_BARBARIAN_LEADER, 0), 0, 0, -1);
   } else {                   /* sea raiders - their units will be veteran */
     struct unit *ptrans;
+    struct unit_type *boat;
 
     barbarians = create_barbarian_player(FALSE);
     boat = find_a_unit_type(L_BARBARIAN_BOAT,-1);
     ptrans = create_unit(barbarians, utile, boat, 0, 0, -1);
-    cap = get_transporter_capacity(unit_list_get(&utile->units, 0));
+    cap = get_transporter_capacity(unit_list_get(utile->units, 0));
     for (i = 0; i < cap-1; i++) {
-      unit = find_a_unit_type(L_BARBARIAN_SEA,L_BARBARIAN_SEA_TECH);
+      struct unit_type *unit
+	= find_a_unit_type(L_BARBARIAN_SEA, L_BARBARIAN_SEA_TECH);
+
       (void) create_unit_full(barbarians, utile, unit, 0, 0, -1, -1,
 			      ptrans);
-      freelog(LOG_DEBUG, "Created barbarian unit %s", unit_types[unit].name);
+      freelog(LOG_DEBUG, "Created barbarian unit %s",
+              utype_rule_name(unit));
     }
     (void) create_unit_full(barbarians, utile,
 			    get_role_unit(L_BARBARIAN_LEADER, 0), 0, 0,
@@ -426,17 +470,19 @@ static void try_summon_barbarians(void)
   } unit_list_iterate_end;
 
   /* to let them know where to get you */
-  show_area(barbarians, utile, 3);
-  show_area(barbarians, pc->tile, 3);
+  map_show_circle(barbarians, utile, BARBARIAN_INITIAL_VISION_RADIUS_SQ);
+  map_show_circle(barbarians, pc->tile, BARBARIAN_INITIAL_VISION_RADIUS_SQ);
 
   /* There should probably be a different message about Sea Raiders */
   if (is_land_barbarian(barbarians)) {
-    notify_player_ex(victim, utile, E_UPRISING,
-		     _("Native unrest near %s led by %s."), pc->name,
-		     barbarians->name);
-  } else if (map_is_known_and_seen(utile, victim)) {
-    notify_player_ex(victim, utile, E_UPRISING,
-		     _("Sea raiders seen near %s!"), pc->name);
+    notify_player(victim, utile, E_UPRISING,
+		  _("Native unrest near %s led by %s."),
+		  city_name(pc),
+		  player_name(barbarians));
+  } else if (map_is_known_and_seen(utile, victim, V_MAIN)) {
+    notify_player(victim, utile, E_UPRISING,
+		  _("Sea raiders seen near %s!"),
+		  city_name(pc));
   }
 }
 
@@ -448,11 +494,11 @@ void summon_barbarians(void)
 {
   int i, n;
 
-  if (game.barbarianrate == 0) {
+  if (game.info.barbarianrate == 0) {
     return;
   }
 
-  if (game.year < game.onsetbarbarian) {
+  if (game.info.year < game.info.onsetbarbarian) {
     return;
   }
 
@@ -462,7 +508,7 @@ void summon_barbarians(void)
     n = 1;
   }
 
-  for (i = 0; i < n * (game.barbarianrate - 1); i++) {
+  for (i = 0; i < n * (game.info.barbarianrate - 1); i++) {
     try_summon_barbarians();
   }
 }
