@@ -207,9 +207,8 @@ Used only in global_warming() and nuclear_winter() below.
 **************************************************************************/
 static bool is_terrain_ecologically_wet(struct tile *ptile)
 {
-  return (tile_has_special(ptile, S_RIVER)
-	  || is_ocean_near_tile(ptile)
-	  || is_special_near_tile(ptile, S_RIVER));
+  return (is_ocean_near_tile(ptile)
+	  || is_special_near_tile(ptile, S_RIVER, TRUE));
 }
 
 /**************************************************************************
@@ -1250,6 +1249,9 @@ void give_shared_vision(struct player *pfrom, struct player *pto)
 	  } vision_layer_iterate_end;
 	} whole_map_iterate_end;
 
+        /* Refresh cities in case new tiles were revealed from their area. */
+        check_city_workers(pplayer2);
+
 	/* squares that are not seen, but which pfrom may have more recent
 	   knowledge of */
 	give_map_from_player_to_player(pplayer, pplayer2);
