@@ -258,7 +258,7 @@ static void finish_revolution(struct player *pplayer)
       = 100 - pplayer->economic.science - pplayer->economic.tax;
   }
 
-  check_player_government_rates(pplayer);
+  check_player_max_rates(pplayer);
   global_city_refresh(pplayer);
   send_player_info(pplayer, pplayer);
 }
@@ -337,7 +337,7 @@ void handle_player_change_government(struct player *pplayer, int government)
 		     _("Revolution: returning to anarchy."));
   }
 
-  check_player_government_rates(pplayer);
+  check_player_max_rates(pplayer);
   global_city_refresh(pplayer);
   send_player_info(pplayer, pplayer);
 
@@ -414,25 +414,22 @@ The following checks that government rates are acceptable for the present
 form of government. Has to be called when switching governments or when
 toggling from AI to human.
 **************************************************************************/
-void check_player_government_rates(struct player *pplayer)
+void check_player_max_rates(struct player *pplayer)
 {
   struct player_economic old_econ = pplayer->economic;
-  bool changed = FALSE;
-  player_limit_to_government_rates(pplayer);
-  if (pplayer->economic.tax != old_econ.tax) {
-    changed = TRUE;
+
+  pplayer->economic = player_limit_to_max_rates(pplayer);
+  if (old_econ.tax > pplayer->economic.tax) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Tax rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
   }
-  if (pplayer->economic.science != old_econ.science) {
-    changed = TRUE;
+  if (old_econ.science > pplayer->economic.science) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Science rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
   }
-  if (pplayer->economic.luxury != old_econ.luxury) {
-    changed = TRUE;
+  if (old_econ.luxury > pplayer->economic.luxury) {
     notify_player(pplayer, NULL, E_NEW_GOVERNMENT,
 		  _("Luxury rate exceeded the max rate for %s; adjusted."), 
 		  government_name_for_player(pplayer));
@@ -842,6 +839,7 @@ static void package_player_common(struct player *plr,
   packet->is_male=plr->is_male;
   packet->team = plr->team ? plr->team->index : -1;
   packet->is_ready = plr->is_ready;
+  packet->was_created = plr->was_created;
   if (city_styles != NULL) {
     packet->city_style = city_style_of_player(plr);
   } else {
@@ -1547,7 +1545,7 @@ static struct player *split_player(struct player *pplayer)
     } players_iterate_end;
   }
 
-  player_limit_to_government_rates(pplayer);
+  pplayer->economic = player_limit_to_max_rates(pplayer);
 
   /* copy the maps */
 
@@ -1652,6 +1650,13 @@ void civil_war(struct player *pplayer)
             "players"), player_name(pplayer));
     return;
   }
+  if (normal_player_count() >= game.playable_nations) {
+    /* No nation for additional player */
+    freelog(LOG_NORMAL,
+            _("Could not throw %s into civil war - no available nations"),
+            nation_plural_for_player(pplayer));
+    return;
+  }
 
   cplayer = split_player(pplayer);
 
@@ -1755,3 +1760,19 @@ void handle_player_phase_done(struct player *pplayer,
 
   send_player_info(pplayer, NULL);
 }
+
+/**************************************************************************
+  Return the number of barbarian players.
+**************************************************************************/
+int barbarian_count(void)
+{
+  return game.info.nbarbarians;
+}
+
+/**************************************************************************
+  Return the number of non-barbarian players.
+**************************************************************************/
+int normal_player_count(void)
+{
+  return game.info.nplayers - game.info.nbarbarians;
+}
