@@ -16,17 +16,20 @@
 #endif
 
 #include "fcintl.h"
+#include "game.h"
+#include "log.h"
 
 #include "map.h"
 
-#include "gamelog.h"
+#include "ggzserver.h"
+#include "plrhand.h"
 #include "report.h"
 #include "settings.h"
+#include "srv_main.h"
 #include "stdinhand.h"
 
 /* Category names must match the values in enum sset_category. */
 const char *sset_category_names[] = {N_("Geological"),
-				     N_("Ecological"),
 				     N_("Sociological"),
 				     N_("Economic"),
 				     N_("Military"),
@@ -39,7 +42,8 @@ const char *sset_level_names[] = {N_("None"),
 				  N_("All"),
 				  N_("Vital"),
 				  N_("Situational"),
-				  N_("Rare")};
+				  N_("Rare"),
+				  N_("Changed")};
 const int OLEVELS_NUM = ARRAY_SIZE(sset_level_names);
 
 
@@ -56,6 +60,7 @@ static bool autotoggle_callback(bool value, const char **reject_message)
   players_iterate(pplayer) {
     if (!pplayer->ai.control && !pplayer->is_connected) {
       toggle_ai_player_direct(NULL, pplayer);
+      send_player_info_c(pplayer, game.est_connections);
     }
   } players_iterate_end;
 
@@ -106,7 +111,7 @@ static bool allowtake_callback(const char *value, const char **error_string)
 
 /*************************************************************************
   Verify that a given startunits string is valid.  See
-  game.start_units.
+  game.info.start_units.
 *************************************************************************/
 static bool startunits_callback(const char *value, const char **error_string)
 {
@@ -122,6 +127,7 @@ static bool startunits_callback(const char *value, const char **error_string)
       have_founder = TRUE;
       continue;
     }
+    /* TODO: add 'f' back in here when we can support ferry units */
     if (strchr("cwxksdDaA", value[i])) {
       continue;
     }
@@ -143,13 +149,35 @@ static bool startunits_callback(const char *value, const char **error_string)
   return TRUE;
 }
 
+/*************************************************************************
+  Verify that a given endyear is valid.
+*************************************************************************/
+static bool endyear_callback(int value, const char **error_string)
+{
+  if (value < game.info.year) {
+    /* Tried to set endyear earlier than current year */
+    *error_string = _("Cannot set endyear earlier than current year.");
+    return FALSE;
+  }
+  return TRUE;
+}
+
 /*************************************************************************
   Verify that a given maxplayers string is valid.
 *************************************************************************/
 static bool maxplayers_callback(int value, const char **error_string)
 {
-  if (value < game.nplayers) {
-    *error_string =_("Number of players is higher than requested value; "
+#ifdef GGZ_SERVER
+  if (with_ggz) {
+    /* In GGZ mode the maxplayers is the number of actual players - set
+     * when the game is lauched and not changed thereafter.  This may be
+     * changed in future. */
+    *error_string = _("Cannot change maxplayers in GGZ mode.");
+    return FALSE;
+  }
+#endif
+  if (value < game.info.nplayers) {
+    *error_string =_("Number of players is higher than requested value;\n"
 		     "Keeping old value.");
     return FALSE;
   }
@@ -158,6 +186,16 @@ static bool maxplayers_callback(int value, const char **error_string)
   return TRUE;
 }
 
+/*************************************************************************
+  Create/remove players when aifill is set.
+*************************************************************************/
+static bool aifill_callback(int value, const char **error_string)
+{
+  aifill(value);
+  error_string = NULL;
+  return TRUE;
+}
+
 #define GEN_BOOL(name, value, sclass, scateg, slevel, to_client,	\
 		 short_help, extra_help, func, default)			\
   {name, sclass, to_client, short_help, extra_help, SSET_BOOL,		\
@@ -366,24 +404,24 @@ struct settings_s settings[] = {
 	  N_("Amount of huts (minor tribe villages)"),
 	  N_("This setting gives the exact number of huts that will be "
 	     "placed on the entire map. Huts are small tribal villages "
-	     "that may be investiged by units."), NULL,
-	  MAP_MIN_HUTS, MAP_MAX_HUTS, MAP_DEFAULT_HUTS)
+	     "that may be investigated by units."),
+	  NULL, MAP_MIN_HUTS, MAP_MAX_HUTS, MAP_DEFAULT_HUTS)
 
   /* Options affecting numbers of players and AI players.  These only
    * affect the start of the game and can not be adjusted after that.
    * (Actually, minplayers does also affect reloads: you can't start a
    * reload game until enough players have connected (or are AI).)
    */
-  GEN_INT("minplayers", game.min_players,
+  GEN_INT("minplayers", game.info.min_players,
 	  SSET_PLAYERS, SSET_INTERNAL, SSET_VITAL,
           SSET_TO_CLIENT,
 	  N_("Minimum number of players"),
 	  N_("There must be at least this many players (connected "
-	     "human players or AI players) before the game can start."),
+	     "human players) before the game can start."),
 	  NULL,
 	  GAME_MIN_MIN_PLAYERS, GAME_MAX_MIN_PLAYERS, GAME_DEFAULT_MIN_PLAYERS)
 
-  GEN_INT("maxplayers", game.max_players,
+  GEN_INT("maxplayers", game.info.max_players,
 	  SSET_PLAYERS, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Maximum number of players"),
           N_("The maximal number of human and AI players who can be in "
@@ -392,19 +430,20 @@ struct settings_s settings[] = {
              "will be rejected."), maxplayers_callback,
 	  GAME_MIN_MAX_PLAYERS, GAME_MAX_MAX_PLAYERS, GAME_DEFAULT_MAX_PLAYERS)
 
-  GEN_INT("aifill", game.aifill,
+  GEN_INT("aifill", game.info.aifill,
 	  SSET_PLAYERS, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Total number of players (including AI players)"),
-	  N_("If there are fewer than this many players when the "
-	     "game starts, extra AI players will be created to "
-	     "increase the total number of players to the value of "
-	     "this option."), NULL, 
+	  N_("If set to a positive value, then AI players will be "
+	     "automatically created or removed to keep the total "
+	     "number of players at this amount."), aifill_callback,
 	  GAME_MIN_AIFILL, GAME_MAX_AIFILL, GAME_DEFAULT_AIFILL)
 
   /* Game initialization parameters (only affect the first start of the game,
    * and not reloads).  Can not be changed after first start of game.
    */
-  GEN_STRING("startunits", game.start_units,
+  /* TODO: Add this line back when we can support Ferry units */
+  /* "    f   = Ferryboat (eg., Trireme)\n" */
+  GEN_STRING("startunits", game.info.start_units,
 	     SSET_GAME_INIT, SSET_SOCIOLOGY, SSET_VITAL, SSET_TO_CLIENT,
              N_("List of players' initial units"),
              N_("This should be a string of characters, each of which "
@@ -422,79 +461,85 @@ struct settings_s settings[] = {
 		"    A   = Strong attack unit (eg., Catapult)\n"),
 		startunits_callback, GAME_DEFAULT_START_UNITS)
 
-  GEN_INT("dispersion", game.dispersion,
+  GEN_INT("dispersion", game.info.dispersion,
 	  SSET_GAME_INIT, SSET_SOCIOLOGY, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	  N_("Area where initial units are located"),
 	  N_("This is the radius within "
 	     "which the initial units are dispersed."), NULL,
 	  GAME_MIN_DISPERSION, GAME_MAX_DISPERSION, GAME_DEFAULT_DISPERSION)
 
-  GEN_INT("gold", game.gold,
+  GEN_INT("gold", game.info.gold,
 	  SSET_GAME_INIT, SSET_ECONOMICS, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Starting gold per player"), 
 	  N_("At the beginning of the game, each player is given this "
 	     "much gold."), NULL,
 	  GAME_MIN_GOLD, GAME_MAX_GOLD, GAME_DEFAULT_GOLD)
 
-  GEN_INT("techlevel", game.tech,
+  GEN_INT("techlevel", game.info.tech,
 	  SSET_GAME_INIT, SSET_SCIENCE, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Number of initial techs per player"), 
 	  N_("At the beginning of the game, each player is given this "
 	     "many technologies. The technologies chosen are random for "
-	     "each player."), NULL,
+	     "each player. Depending on the value of tech_cost_style in "
+             "the ruleset, a big value for techlevel can make the next "
+             "techs really expensive."), NULL,
 	  GAME_MIN_TECHLEVEL, GAME_MAX_TECHLEVEL, GAME_DEFAULT_TECHLEVEL)
 
-  GEN_INT("researchcost", game.researchcost,
+  GEN_INT("sciencebox", game.info.sciencebox,
 	  SSET_RULES, SSET_SCIENCE, SSET_SITUATIONAL, SSET_TO_CLIENT,
-	  N_("Points required to gain a new tech"),
+	  N_("Technology cost multiplier percentage"),
 	  N_("This affects how quickly players can research new "
-	     "technology. Doubling its value will make all technologies "
-	     "take twice as long to research."), NULL,
-	  GAME_MIN_RESEARCHCOST, GAME_MAX_RESEARCHCOST, 
-	  GAME_DEFAULT_RESEARCHCOST)
+	     "technology. All tech costs are multiplied by this amount "
+	     "(as a percentage). The base tech costs are determined by "
+	     "the ruleset or other game settings."),
+	  NULL, GAME_MIN_SCIENCEBOX, GAME_MAX_SCIENCEBOX, 
+	  GAME_DEFAULT_SCIENCEBOX)
 
-  GEN_INT("techpenalty", game.techpenalty,
+  GEN_INT("techpenalty", game.info.techpenalty,
 	  SSET_RULES, SSET_SCIENCE, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Percentage penalty when changing tech"),
 	  N_("If you change your current research technology, and you have "
 	     "positive research points, you lose this percentage of those "
-	     "research points. This does not apply if you have just gained "
+	     "research points. This does not apply when you have just gained "
 	     "a technology this turn."), NULL,
 	  GAME_MIN_TECHPENALTY, GAME_MAX_TECHPENALTY,
 	  GAME_DEFAULT_TECHPENALTY)
 
-  GEN_INT("diplcost", game.diplcost,
+  GEN_INT("diplcost", game.info.diplcost,
 	  SSET_RULES, SSET_SCIENCE, SSET_RARE, SSET_TO_CLIENT,
-	  N_("Penalty when getting tech from treaty"),
+	  N_("Penalty when getting tech or gold from treaty"),
 	  N_("For each technology you gain from a diplomatic treaty, you "
-	     "lost research points equal to this percentage of the cost to "
-	     "research a new technology. You can end up with negative "
-	     "research points if this is non-zero."), NULL, 
+	     "lose research points equal to this percentage of the cost to "
+	     "research a new technology. If this is non-zero, you can end up "
+	     "with negative research points. Also applies to gold "
+             "transfers in diplomatic treaties."),
+          NULL,
 	  GAME_MIN_DIPLCOST, GAME_MAX_DIPLCOST, GAME_DEFAULT_DIPLCOST)
 
-  GEN_INT("conquercost", game.conquercost,
+  GEN_INT("conquercost", game.info.conquercost,
 	  SSET_RULES, SSET_SCIENCE, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Penalty when getting tech from conquering"),
 	  N_("For each technology you gain by conquering an enemy city, you "
-	     "lose research points equal to this percentage of the cost "
-	     "to research a new technology. You can end up with negative "
-	     "research points if this is non-zero."), NULL,
+	     "lose research points equal to this percentage of the cost to "
+	     "research a new technology. If this is non-zero, you can end up "
+	     "with negative research points."),
+	  NULL,
 	  GAME_MIN_CONQUERCOST, GAME_MAX_CONQUERCOST,
 	  GAME_DEFAULT_CONQUERCOST)
 
-  GEN_INT("freecost", game.freecost,
+  GEN_INT("freecost", game.info.freecost,
 	  SSET_RULES, SSET_SCIENCE, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Penalty when getting a free tech"),
 	  N_("For each technology you gain \"for free\" (other than "
 	     "covered by diplcost or conquercost: specifically, from huts "
 	     "or from Great Library effects), you lose research points "
 	     "equal to this percentage of the cost to research a new "
-	     "technology. You can end up with negative research points if "
-	     "this is non-zero."), 
+	     "technology. If this is non-zero, you can end up "
+	     "with negative research points."), 
 	  NULL, 
 	  GAME_MIN_FREECOST, GAME_MAX_FREECOST, GAME_DEFAULT_FREECOST)
 
-  GEN_INT("foodbox", game.foodbox,
+  GEN_INT("foodbox", game.info.foodbox,
 	  SSET_RULES, SSET_ECONOMICS, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	  N_("Food required for a city to grow"),
 	  N_("This is the base amount of food required to grow a city. "
@@ -503,23 +548,32 @@ struct settings_s settings[] = {
 	  NULL,
 	  GAME_MIN_FOODBOX, GAME_MAX_FOODBOX, GAME_DEFAULT_FOODBOX)
 
-  GEN_INT("aqueductloss", game.aqueductloss,
+  GEN_INT("aqueductloss", game.info.aqueductloss,
 	  SSET_RULES, SSET_ECONOMICS, SSET_RARE, SSET_TO_CLIENT,
-	  N_("Percentage food lost when need aqueduct"),
+	  N_("Percentage food lost when building needed"),
 	  N_("If a city would expand, but it can't because it needs "
 	     "an Aqueduct (or Sewer System), it loses this percentage "
-	     "of its foodbox (or half that amount if it has a "
+	     "of its foodbox (or half that amount when it has a "
 	     "Granary)."), NULL, 
 	  GAME_MIN_AQUEDUCTLOSS, GAME_MAX_AQUEDUCTLOSS, 
 	  GAME_DEFAULT_AQUEDUCTLOSS)
 
+  GEN_INT("shieldbox", game.info.shieldbox,
+	  SSET_RULES, SSET_ECONOMICS, SSET_SITUATIONAL, SSET_TO_CLIENT,
+	  N_("Multiplier percentage for production costs"),
+	  N_("This affects how quickly units and buildings can be "
+	     "produced.  The base costs are multiplied by this value (as "
+	     "a percentage)."),
+	  NULL, GAME_MIN_SHIELDBOX, GAME_MAX_SHIELDBOX,
+	  GAME_DEFAULT_SHIELDBOX)
+
   /* Notradesize and fulltradesize used to have callbacks to prevent them
    * from being set illegally (notradesize > fulltradesize).  However this
    * provided a problem when setting them both through the client's settings
    * dialog, since they cannot both be set atomically.  So the callbacks were
    * removed and instead the game now knows how to deal with invalid
    * settings. */
-  GEN_INT("fulltradesize", game.fulltradesize,
+  GEN_INT("fulltradesize", game.info.fulltradesize,
 	  SSET_RULES, SSET_ECONOMICS, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Minimum city size to get full trade"),
 	  N_("There is a trade penalty in all cities smaller than this. "
@@ -530,7 +584,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_FULLTRADESIZE, GAME_MAX_FULLTRADESIZE, 
 	  GAME_DEFAULT_FULLTRADESIZE)
 
-  GEN_INT("notradesize", game.notradesize,
+  GEN_INT("notradesize", game.info.notradesize,
 	  SSET_RULES, SSET_ECONOMICS, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Maximum size of a city without trade"),
 	  N_("Cities do not produce any trade at all unless their size "
@@ -540,7 +594,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_NOTRADESIZE, GAME_MAX_NOTRADESIZE,
 	  GAME_DEFAULT_NOTRADESIZE)
 
-  GEN_INT("unhappysize", game.unhappysize,
+  GEN_INT("unhappysize", game.info.unhappysize,
 	  SSET_RULES, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("City size before people become unhappy"),
 	  N_("Before other adjustments, the first unhappysize citizens in a "
@@ -549,16 +603,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_UNHAPPYSIZE, GAME_MAX_UNHAPPYSIZE,
 	  GAME_DEFAULT_UNHAPPYSIZE)
 
-  GEN_BOOL("angrycitizen", game.angrycitizen,
-	   SSET_RULES, SSET_SOCIOLOGY, SSET_SITUATIONAL, SSET_TO_CLIENT,
-	  N_("Whether angry citizens are enabled"),
-	  N_("Introduces angry citizens like in civilization II. Angry "
-	     "citizens have to become unhappy before any other class "
-	     "of citizens may be considered. See also unhappysize, "
-	     "cityfactor and governments."), NULL, 
-	  GAME_DEFAULT_ANGRYCITIZEN)
-
-  GEN_INT("cityfactor", game.cityfactor,
+  GEN_INT("cityfactor", game.info.cityfactor,
 	  SSET_RULES, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Number of cities for higher unhappiness"),
 	  N_("When the number of cities a player owns is greater than "
@@ -568,19 +613,19 @@ struct settings_s settings[] = {
 	     "smaller numbers of cities."), NULL, 
 	  GAME_MIN_CITYFACTOR, GAME_MAX_CITYFACTOR, GAME_DEFAULT_CITYFACTOR)
 
-  GEN_INT("citymindist", game.citymindist,
+  GEN_INT("citymindist", game.info.citymindist,
 	  SSET_RULES, SSET_SOCIOLOGY, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	  N_("Minimum distance between cities"),
-	  N_("When a player founds a new city, it is checked if there is "
-	     "no other city in citymindist distance. For example, if "
-	     "citymindist is 3, there have to be at least two empty "
+	  N_("When a player attempts to found a new city, there may be "
+	     "no other city in this distance. For example, when "
+	     "this value is 3, there have to be at least two empty "
 	     "fields between two cities in every direction. If set "
 	     "to 0 (default), the ruleset value will be used."),
 	  NULL,
 	  GAME_MIN_CITYMINDIST, GAME_MAX_CITYMINDIST,
 	  GAME_DEFAULT_CITYMINDIST)
 
-  GEN_INT("rapturedelay", game.rapturedelay,
+  GEN_INT("rapturedelay", game.info.rapturedelay,
 	  SSET_RULES, SSET_SOCIOLOGY, SSET_SITUATIONAL, SSET_TO_CLIENT,
           N_("Number of turns between rapture effect"),
           N_("Sets the number of turns between rapture growth of a city. "
@@ -590,36 +635,36 @@ struct settings_s settings[] = {
           GAME_MIN_RAPTUREDELAY, GAME_MAX_RAPTUREDELAY,
           GAME_DEFAULT_RAPTUREDELAY)
 
-  GEN_INT("razechance", game.razechance,
+  GEN_INT("razechance", game.info.razechance,
 	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Chance for conquered building destruction"),
 	  N_("When a player conquers a city, each city improvement has this "
 	     "percentage chance to be destroyed."), NULL, 
 	  GAME_MIN_RAZECHANCE, GAME_MAX_RAZECHANCE, GAME_DEFAULT_RAZECHANCE)
 
-  GEN_INT("civstyle", game.civstyle,
-	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
-          N_("civstyle is an obsolete setting"),
-          N_("This setting is obsolete; it does nothing in the current "
-	     "version. It will be removed from future versions."), NULL,
-	  GAME_MIN_CIVSTYLE, GAME_MAX_CIVSTYLE, GAME_DEFAULT_CIVSTYLE)
-
-  GEN_INT("occupychance", game.occupychance,
+  GEN_INT("occupychance", game.info.occupychance,
 	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Chance of moving into tile after attack"),
 	  N_("If set to 0, combat is Civ1/2-style (when you attack, "
 	     "you remain in place). If set to 100, attacking units "
-	     "will always move into the tile they attacked if they win "
+	     "will always move into the tile they attacked when they win "
 	     "the combat (and no enemy units remain in the tile). If "
 	     "set to a value between 0 and 100, this will be used as "
 	     "the percent chance of \"occupying\" territory."), NULL, 
 	  GAME_MIN_OCCUPYCHANCE, GAME_MAX_OCCUPYCHANCE, 
 	  GAME_DEFAULT_OCCUPYCHANCE)
 
-  GEN_INT("killcitizen", game.killcitizen,
+  GEN_BOOL("autoattack", game.info.autoattack, SSET_RULES_FLEXIBLE, SSET_MILITARY,
+         SSET_SITUATIONAL, SSET_TO_CLIENT,
+         N_("Turn on/off server-side autoattack"),
+         N_("If set to on, units with move left will automatically "
+            "consider attacking enemy units that move adjacent to them."), 
+         NULL, GAME_DEFAULT_AUTOATTACK)
+
+  GEN_INT("killcitizen", game.info.killcitizen,
 	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Reduce city population after attack"),
-	  N_("This flag indicates if city population is reduced "
+	  N_("This flag indicates whether city population is reduced "
 	     "after successful attack of enemy unit, depending on "
 	     "its movement type (OR-ed):\n"
 	     "  1 = land\n"
@@ -629,32 +674,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_KILLCITIZEN, GAME_MAX_KILLCITIZEN,
 	  GAME_DEFAULT_KILLCITIZEN)
 
-  GEN_INT("wtowervision", game.watchtower_vision,
-	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
-	  N_("Range of vision for units in a fortress"),
-	  N_("If set to 1, it has no effect. "
-	     "If 2 or larger, the vision range of a unit inside a "
-	     "fortress is set to this value, if the necessary invention "
-	     "has been made. This invention is determined by the flag "
-	     "'Watchtower' in the techs ruleset. See also wtowerevision."), 
-	  NULL, 
-	  GAME_MIN_WATCHTOWER_VISION, GAME_MAX_WATCHTOWER_VISION, 
-	  GAME_DEFAULT_WATCHTOWER_VISION)
-
-  GEN_INT("wtowerevision", game.watchtower_extra_vision,
-	  SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
-	  N_("Extra vision range for units in a fortress"),
-	  N_("If set to 0, it has no "
-	     "effect. If larger than 0, the vision range of a unit is "
-	     "raised by this value, if the unit is inside a fortress "
-	     "and the invention determined by the flag 'Watchtower' "
-	     "in the techs ruleset has been made. Always the larger "
-	     "value of wtowervision and wtowerevision will be used. "
-	     "Also see wtowervision."), NULL, 
-	  GAME_MIN_WATCHTOWER_EXTRA_VISION, GAME_MAX_WATCHTOWER_EXTRA_VISION, 
-	  GAME_DEFAULT_WATCHTOWER_EXTRA_VISION)
-
-  GEN_INT("borders", game.borders,
+  GEN_INT("borders", game.info.borders,
 	  SSET_RULES, SSET_MILITARY, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	  N_("National borders radius"),
 	  N_("If this is set to greater than 0, then any land tiles "
@@ -664,7 +684,7 @@ struct settings_s settings[] = {
 	  NULL,
 	  GAME_MIN_BORDERS, GAME_MAX_BORDERS, GAME_DEFAULT_BORDERS)
 
-  GEN_BOOL("happyborders", game.happyborders,
+  GEN_BOOL("happyborders", game.info.happyborders,
 	   SSET_RULES, SSET_MILITARY, SSET_SITUATIONAL,
 	   SSET_TO_CLIENT,
 	   N_("Units inside borders cause no unhappiness"),
@@ -672,18 +692,17 @@ struct settings_s settings[] = {
 	      "inside your own borders."), NULL,
 	   GAME_DEFAULT_HAPPYBORDERS)
 
-  GEN_INT("diplomacy", game.diplomacy,
+  GEN_INT("diplomacy", game.info.diplomacy,
 	  SSET_RULES, SSET_MILITARY, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	  N_("Ability to do diplomacy with other players"),
-	  N_("0 = default; diplomacy is enabled for everyone.\n\n"
-	     "1 = diplomacy is only allowed between human players.\n\n"
-	     "2 = diplomacy is only allowed between AI players.\n\n"
-             "3 = diplomacy is restricted to teams.\n\n"
-             "4 = diplomacy is disabled for everyone.\n\n"
-             "You can always do diplomacy with players on your team."), NULL,
+	  N_("0 = default; diplomacy is enabled for everyone.\n"
+	     "1 = diplomacy is only allowed between human players.\n"
+	     "2 = diplomacy is only allowed between AI players.\n"
+             "3 = diplomacy is restricted to teams.\n"
+             "4 = diplomacy is disabled for everyone."), NULL,
 	  GAME_MIN_DIPLOMACY, GAME_MAX_DIPLOMACY, GAME_DEFAULT_DIPLOMACY)
 
-  GEN_INT("citynames", game.allowed_city_names,
+  GEN_INT("citynames", game.info.allowed_city_names,
 	  SSET_RULES, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Allowed city names"),
 	  N_("0 = There are no restrictions: players can have "
@@ -712,7 +731,7 @@ struct settings_s settings[] = {
    *      packet_game_info) should probably not be flexible, or at
    *      least need extra care to be flexible.
    */
-  GEN_INT("barbarians", game.barbarianrate,
+  GEN_INT("barbarians", game.info.barbarianrate,
 	  SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Barbarian appearance frequency"),
 	  N_("0 = no barbarians \n"
@@ -723,14 +742,14 @@ struct settings_s settings[] = {
 	  GAME_MIN_BARBARIANRATE, GAME_MAX_BARBARIANRATE, 
 	  GAME_DEFAULT_BARBARIANRATE)
 
-  GEN_INT("onsetbarbs", game.onsetbarbarian,
+  GEN_INT("onsetbarbs", game.info.onsetbarbarian,
 	  SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Barbarian onset year"),
 	  N_("Barbarians will not appear before this year."), NULL,
 	  GAME_MIN_ONSETBARBARIAN, GAME_MAX_ONSETBARBARIAN, 
 	  GAME_DEFAULT_ONSETBARBARIAN)
 
-  GEN_INT("revolen", game.revolution_length,
+  GEN_INT("revolen", game.info.revolution_length,
 	  SSET_RULES_FLEXIBLE, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Length in turns of revolution"),
 	  N_("When changing governments, a period of anarchy lasting this "
@@ -740,8 +759,8 @@ struct settings_s settings[] = {
 	  GAME_MIN_REVOLUTION_LENGTH, GAME_MAX_REVOLUTION_LENGTH, 
 	  GAME_DEFAULT_REVOLUTION_LENGTH)
 
-  GEN_BOOL("fogofwar", game.fogofwar,
-	   SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
+  GEN_BOOL("fogofwar", game.info.fogofwar,
+	   SSET_RULES, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
 	   N_("Whether to enable fog of war"),
 	   N_("If this is set to 1, only those units and cities within "
 	      "the vision range of your own units and cities will be "
@@ -749,61 +768,54 @@ struct settings_s settings[] = {
 	      "changes in tiles not observed."), NULL, 
 	   GAME_DEFAULT_FOGOFWAR)
 
-  GEN_INT("diplchance", game.diplchance,
+  GEN_INT("diplchance", game.info.diplchance,
 	  SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_SITUATIONAL, SSET_TO_CLIENT,
-	  N_("Chance in diplomat/spy contests"),
+	  N_("Base chance for diplomats and spies to succeed."),
 	  /* xgettext:no-c-format */
-	  N_("A diplomatic unit acting against a city which has one or "
-	     "more defending diplomatic units has a diplchance "
-	     "(percent) chance to defeat each such defender. Also, the "
-	     "chance of a spy returning from a successful mission is "
-	     "diplchance percent (diplomats never return).  This value is "
-	     "also the basic chance of success for diplomats and spies. "
-	     "Defending spies are generally twice as capable as "
-	     "diplomats; veteran units are 50% more capable than "
-	     "non-veteran ones."), NULL, 
+	  N_("The chance of a spy returning from a successful mission and "
+	     "the base chance of success for diplomats and spies."),
+          NULL,
 	  GAME_MIN_DIPLCHANCE, GAME_MAX_DIPLCHANCE, GAME_DEFAULT_DIPLCHANCE)
 
-  GEN_BOOL("spacerace", game.spacerace,
+  GEN_BOOL("spacerace", game.info.spacerace,
 	   SSET_RULES_FLEXIBLE, SSET_SCIENCE, SSET_VITAL, SSET_TO_CLIENT,
 	   N_("Whether to allow space race"),
 	   N_("If this option is set to 1, players can build spaceships."),
 	   NULL, 
 	   GAME_DEFAULT_SPACERACE)
 
-  GEN_INT("civilwarsize", game.civilwarsize,
+  GEN_INT("civilwarsize", game.info.civilwarsize,
 	  SSET_RULES_FLEXIBLE, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Minimum number of cities for civil war"),
-	  N_("A civil war is triggered if a player has at least this "
+	  N_("A civil war is triggered when a player has at least this "
 	     "many cities and the player's capital is captured. If "
 	     "this option is set to the maximum value, civil wars are "
 	     "turned off altogether."), NULL, 
 	  GAME_MIN_CIVILWARSIZE, GAME_MAX_CIVILWARSIZE, 
 	  GAME_DEFAULT_CIVILWARSIZE)
 
-  GEN_INT("contactturns", game.contactturns,
+  GEN_INT("contactturns", game.info.contactturns,
 	  SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Turns until player contact is lost"),
 	  N_("Players may meet for diplomacy this number of turns "
-	     "after their units have last met, even if they do not have "
-	     "an embassy. If set to zero then players cannot meet unless "
+	     "after their units have last met, even when they do not have "
+	     "an embassy. If set to zero, then players cannot meet unless "
 	     "they have an embassy."),
 	  NULL,
 	  GAME_MIN_CONTACTTURNS, GAME_MAX_CONTACTTURNS, 
 	  GAME_DEFAULT_CONTACTTURNS)
 
-  GEN_BOOL("savepalace", game.savepalace,
+  GEN_BOOL("savepalace", game.info.savepalace,
 	   SSET_RULES_FLEXIBLE, SSET_MILITARY, SSET_RARE, SSET_TO_CLIENT,
-	   N_("Rebuild palace if capital is conquered"),
-	   N_("If this is set to 1, then when the capital is conquered the "
-	      "palace "
-	      "is automatically rebuilt for free in another randomly "
+	   N_("Rebuild palace whenever capital is conquered"),
+	   N_("If this is set to 1, when the capital is conquered the "
+	      "palace is automatically rebuilt for free in another randomly "
 	      "choosen city. This is significant because the technology "
 	      "requirement for building a palace will be ignored."),
 	   NULL,
 	   GAME_DEFAULT_SAVEPALACE)
 
-  GEN_BOOL("naturalcitynames", game.natural_city_names,
+  GEN_BOOL("naturalcitynames", game.info.natural_city_names,
            SSET_RULES_FLEXIBLE, SSET_SOCIOLOGY, SSET_RARE, SSET_TO_CLIENT,
            N_("Whether to use natural city names"),
            N_("If enabled, the default city names will be determined based "
@@ -842,7 +854,7 @@ struct settings_s settings[] = {
                 "of connection:\n\n"
                 "(none) = Controller allowed, observers allowed, "
                 "can displace connections. (Displacing a connection means "
-		"that you may take over a player, even if another user "
+		"that you may take over a player, even when another user "
 		"already controls that player.)\n\n"
                 "1 = Controller allowed, observers allowed, "
                 "can't displace connections;\n\n"
@@ -853,20 +865,21 @@ struct settings_s settings[] = {
                 "4 = No controller allowed, observers allowed;\n\n"),
                 allowtake_callback, GAME_DEFAULT_ALLOW_TAKE)
 
-  GEN_BOOL("autotoggle", game.auto_ai_toggle,
+  GEN_BOOL("autotoggle", game.info.auto_ai_toggle,
 	   SSET_META, SSET_NETWORK, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	   N_("Whether AI-status toggles with connection"),
 	   N_("If this is set to 1, AI status is turned off when a player "
 	      "connects, and on when a player disconnects."),
 	   autotoggle_callback, GAME_DEFAULT_AUTO_AI_TOGGLE)
 
-  GEN_INT("endyear", game.end_year,
-	  SSET_META, SSET_SOCIOLOGY, SSET_TO_CLIENT, SSET_VITAL,
-	  N_("Year the game ends"), 
-	  N_("The game will end at the end of the given year."), NULL,
-	  GAME_MIN_END_YEAR, GAME_MAX_END_YEAR, GAME_DEFAULT_END_YEAR)
+  GEN_INT("endyear", game.info.end_year,
+	  SSET_META, SSET_SOCIOLOGY, SSET_VITAL, SSET_TO_CLIENT,
+	  N_("Year the game ends"),
+          N_("The game will end at the end of the given year."),
+          endyear_callback,
+          GAME_MIN_END_YEAR, GAME_MAX_END_YEAR, GAME_DEFAULT_END_YEAR)
 
-  GEN_INT("timeout", game.timeout,
+  GEN_INT("timeout", game.info.timeout,
 	  SSET_META, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Maximum seconds per turn"),
 	  N_("If all players have not hit \"Turn Done\" before this "
@@ -878,13 +891,24 @@ struct settings_s settings[] = {
 	   GAME_MIN_TIMEOUT, GAME_MAX_TIMEOUT, GAME_DEFAULT_TIMEOUT)
 
   GEN_INT("timeaddenemymove", game.timeoutaddenemymove,
-        SSET_META, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
+	  SSET_META, SSET_INTERNAL, SSET_VITAL, SSET_TO_CLIENT,
 	  N_("Timeout at least n seconds when enemy moved"),
-	  N_("Any time a unit moves when in sight of an enemy player, "
-	     "the remaining timeout is set to this value if it was lower."),
+	  N_("Any time a unit moves while in sight of an enemy player, "
+	     "the remaining timeout is increased to this value."),
 	  NULL, 0, GAME_MAX_TIMEOUT, GAME_DEFAULT_TIMEOUTADDEMOVE)
   
-  GEN_INT("nettimeout", game.tcptimeout,
+  /* This setting points to the "stored" value; changing it won't have
+   * an effect until the next synchronization point (i.e., the start of
+   * the next turn). */
+  GEN_BOOL("simultaneousphases", game.simultaneous_phases_stored,
+	   SSET_META, SSET_INTERNAL, SSET_SITUATIONAL, SSET_TO_CLIENT,
+	   N_("Whether to have simultaneous player phases."),
+	   N_("If true, all players' movement phases will occur "
+	      "simultaneously; if false, then players will "
+	      "alternate movement."), NULL,
+	   GAME_DEFAULT_SIMULTANEOUS_PHASES)
+
+  GEN_INT("nettimeout", game.info.tcptimeout,
 	  SSET_META, SSET_NETWORK, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Seconds to let a client's network connection block"),
 	  N_("If a network connection is blocking for a time greater than "
@@ -894,7 +918,7 @@ struct settings_s settings[] = {
 	  NULL,
 	  GAME_MIN_TCPTIMEOUT, GAME_MAX_TCPTIMEOUT, GAME_DEFAULT_TCPTIMEOUT)
 
-  GEN_INT("netwait", game.netwait,
+  GEN_INT("netwait", game.info.netwait,
 	  SSET_META, SSET_NETWORK, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Max seconds for network buffers to drain"),
 	  N_("The server will wait for up to the value of this "
@@ -903,14 +927,14 @@ struct settings_s settings[] = {
 	     "wait at all."), NULL, 
 	  GAME_MIN_NETWAIT, GAME_MAX_NETWAIT, GAME_DEFAULT_NETWAIT)
 
-  GEN_INT("pingtime", game.pingtime,
+  GEN_INT("pingtime", game.info.pingtime,
 	  SSET_META, SSET_NETWORK, SSET_RARE, SSET_TO_CLIENT,
 	  N_("Seconds between PINGs"),
 	  N_("The civserver will poll the clients with a PING request "
 	     "each time this period elapses."), NULL, 
 	  GAME_MIN_PINGTIME, GAME_MAX_PINGTIME, GAME_DEFAULT_PINGTIME)
 
-  GEN_INT("pingtimeout", game.pingtimeout,
+  GEN_INT("pingtimeout", game.info.pingtimeout,
 	  SSET_META, SSET_NETWORK, SSET_RARE,
           SSET_TO_CLIENT,
 	  N_("Time to cut a client"),
@@ -918,7 +942,7 @@ struct settings_s settings[] = {
 	     "client is disconnected."), NULL, 
 	  GAME_MIN_PINGTIMEOUT, GAME_MAX_PINGTIMEOUT, GAME_DEFAULT_PINGTIMEOUT)
 
-  GEN_BOOL("turnblock", game.turnblock,
+  GEN_BOOL("turnblock", game.info.turnblock,
 	   SSET_META, SSET_INTERNAL, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	   N_("Turn-blocking game play mode"),
 	   N_("If this is set to 1 the game turn is not advanced "
@@ -926,11 +950,11 @@ struct settings_s settings[] = {
 	      "disconnected players."), NULL, 
 	   GAME_DEFAULT_TURNBLOCK)
 
-  GEN_BOOL("fixedlength", game.fixedlength,
+  GEN_BOOL("fixedlength", game.info.fixedlength,
 	   SSET_META, SSET_INTERNAL, SSET_SITUATIONAL, SSET_TO_CLIENT,
 	   N_("Fixed-length turns play mode"),
 	   N_("If this is set to 1 the game turn will not advance "
-	      "until the timeout has expired, even if all players "
+	      "until the timeout has expired, even after all players "
 	      "have clicked on \"Turn Done\"."), NULL,
 	   FALSE)
 
@@ -959,18 +983,18 @@ struct settings_s settings[] = {
 		"their capitalization is."),
 	     is_valid_demography, GAME_DEFAULT_DEMOGRAPHY)
 
-  GEN_INT("saveturns", game.save_nturns,
+  GEN_INT("saveturns", game.info.save_nturns,
 	  SSET_META, SSET_INTERNAL, SSET_VITAL, SSET_SERVER_ONLY,
 	  N_("Turns per auto-save"),
 	  N_("The game will be automatically saved per this number of "
 	     "turns. Zero means never auto-save."), NULL, 
-	  0, 200, 10)
+	  0, 200, GAME_DEFAULT_SAVETURNS)
 
   /* Could undef entire option if !HAVE_LIBZ, but this way users get to see
    * what they're missing out on if they didn't compile with zlib?  --dwp
    */
 #ifdef HAVE_LIBZ
-  GEN_INT("compress", game.save_compress_level,
+  GEN_INT("compress", game.info.save_compress_level,
 	  SSET_META, SSET_INTERNAL, SSET_RARE, SSET_SERVER_ONLY,
 	  N_("Savegame compression level"),
 	  N_("If non-zero, saved games will be compressed using zlib "
@@ -981,7 +1005,7 @@ struct settings_s settings[] = {
 	  GAME_MIN_COMPRESS_LEVEL, GAME_MAX_COMPRESS_LEVEL,
 	  GAME_DEFAULT_COMPRESS_LEVEL)
 #else
-  GEN_INT("compress", game.save_compress_level,
+  GEN_INT("compress", game.info.save_compress_level,
 	  SSET_META, SSET_INTERNAL, SSET_RARE, SSET_SERVER_ONLY,
 	  N_("Savegame compression level"),
 	  N_("If non-zero, saved games will be compressed using zlib "
@@ -1009,17 +1033,17 @@ struct settings_s settings[] = {
 	      "can be used to create power graphs after the game."), NULL,
 	   GAME_DEFAULT_SCORELOG)
 
-  GEN_INT("gamelog", gamelog_level,
-	  SSET_META, SSET_INTERNAL, SSET_SITUATIONAL, SSET_SERVER_ONLY,
-	  N_("Detail level for logging game events"),
-	  N_("Only applies if the game log feature is enabled "
-	     "(with the -g command line option). "
-	     "Levels: 0=no logging, 20=standard logging, 30=detailed "
-	     "logging, 40=debuging logging."), NULL,
-	  0, 40, 20)
-
   GEN_END
 };
 
 /* The number of settings, not including the END. */
 const int SETTINGS_NUM = ARRAY_SIZE(settings) - 1;
+
+/****************************************************************************
+  Returns whether the specified server setting (option) can currently
+  be changed.  Does not indicate whether it can be changed by clients.
+****************************************************************************/
+bool setting_is_changeable(int setting_id)
+{
+  return setting_class_is_changeable(settings[setting_id].sclass);
+}
