@@ -934,6 +934,8 @@ void save_game(char *orig_filename, const char *save_reason)
   freelog(LOG_VERBOSE, "Save time: %g seconds (%g apparent)",
 	  read_timer_seconds_free(timer_cpu),
 	  read_timer_seconds_free(timer_user));
+
+  ggz_game_saved(filename);
 }
 
 /**************************************************************************
@@ -987,12 +989,17 @@ void server_quit(void)
   set_server_state(S_S_OVER);
   server_game_free();
   diplhand_free();
+
+  /* Free all the warmap arrays */
+  free_mapqueue();
+
 #ifdef HAVE_AUTH
   if (srvarg.auth_enabled) {
     /* If auth has been initialized */
     auth_free();
   }
 #endif /* HAVE_AUTH */
+
   stdinhand_free();
   close_connections_and_socket();
   exit(EXIT_SUCCESS);
@@ -2064,8 +2071,8 @@ static void srv_ready(void)
 
     players_iterate(pplayer) {
       player_map_allocate(pplayer);
-      init_tech(pplayer);
-      player_limit_to_government_rates(pplayer);
+      init_tech(pplayer, TRUE);
+      pplayer->economic = player_limit_to_max_rates(pplayer);
       pplayer->economic.gold = game.info.gold;
     } players_iterate_end;
 
